/*
    TDMB412 driver for Linux -- tdmb412-const.h
    Brian Hughes
    June, 1995
    See ChangeLog for revision history

    A bunch of constants for the tdmb412 driver.  
*/

#ifndef TDMB412_CONSTANTS_H
#define TDMB412_CONSTANTS_H


/* 
    User configurable constants.  
*/

#define DEVICE_NAME        "tdmb412" /* Name of device */
#define MAJOR_DEVICE_NUM    34	     /* Major device number  */
#define DRIVER_VERSION      "0.5"    /* Version of device driver */
#define ADDRESS_EXTENT      0x20     /* Length of board's address space */
#define IO_BASE_DEFAULT     0x200    /* Default IO base addr. of card */
#define JTAG_IRQ_DEFAULT    0x0	     /* Default IRQ for JTAG (0 = unused) */
#define COMPORT_IRQ_DEFAULT 0x0	     /* Default IRQ for comport (0 = unused) */

/* 
    Register offsets from base address.  These will be added to the
    base address of the card below (after it is either autodetected or
    given on a command line) 
*/

#define RXDATA_8_OFFSET      0x00        /* 8-bit receive data port */
#define TXDATA_8_OFFSET      0x01	     /* 8-bit transmit data port */
#define RXSTAT_8_OFFSET      0x02	     /* 8-bit receive status */
#define TXSTAT_8_OFFSET      0x03	     /* 8-bit transmit status */
#define RESET_OFFSET         0x06	     /* Reset */
#define CONFIG_SENSE_OFFSET  0x08	     /* Config/Sense port */
#define RXDATA_16_OFFSET     0x10	     /* 16-bit receive data */
#define TXDATA_16_OFFSET     0x12	     /* 16-bit transmit data */
#define RXSTAT_16_OFFSET     0x14	     /* 16-bit receive status */
#define TXSTAT_16_OFFSET     0x16	     /* 16-bit transmit status */
#define INT_MASK_OFFSET      0x18	     /* Interrupt mask */

/* 
    Actual register locations
*/

#define RXDATA_8_REG     (io_base + RXDATA_8_OFFSET)   /* 8-bit rcv data */
#define TXDATA_8_REG     (io_base + TXDATA_8_OFFSET)   /* 8-bit xmit data */
#define RXSTAT_8_REG     (io_base + RXSTAT_8_OFFSET)   /* 8-bit rcv status */
#define TXSTAT_8_REG     (io_base + TXSTAT_8_OFFSET)   /* 8-bit xmit status */
#define RESET_REG        (io_base + RESET_OFFSET)      /* reset */
#define CONFIG_SENSE_REG (io_base + CONFIG_SENSE_OFFSET)/* Config/Sense */
#define RXDATA_16_REG    (io_base + RXDATA_16_OFFSET)  /* 16-bit rcv data */
#define TXDATA_16_REG    (io_base + TXDATA_16_OFFSET)  /* 16-bit xmit data */
#define RXSTAT_16_REG    (io_base + RXSTAT_16_OFFSET)  /* 16-bit rcv status */
#define TXSTAT_16_REG    (io_base + TXSTAT_16_OFFSET)  /* 16-bit xmit status */
#define INT_MASK_REG     (io_base + INT_MASK_OFFSET)   /* interrupt mask */

/* 
    Bit masks for config/sense port
*/

#define SENSE_MASK      0xf0           /* Mask to isolate SENSE bits */
#define CONFIG_MASK     0x0f	       /* Mask to isolate CONFIG bits */

/*
    Bit masks for 16-bit receive status register
*/

#define RXSTAT_16_DP    0x0080	       /* There is some data in the FIFO*/
#define RXSTAT_16_HF    0x0040	       /* 512 bytes or more in FIFO */
#define RXSTAT_16_AF    0x0020	       /* 256 bytes from full or empty */
#define RXSTAT_16_NF    0x0010	       /* Space available in FIFO */
#define RXSTAT_16_ALL   0x00f0	       /* All the above */

/*
    Bit masks for 16-bit transmit status register
*/


#define TXSTAT_16_OR    0x0080	       /* Output ready */
#define TXSTAT_16_HF    0x0040	       /* 512 bytes in FIFO */
#define TXSTAT_16_AF    0x0020	       /* 256 bytes away from full or empty */
#define TXSTAT_16_NE    0x0010	       /* At least one word in FIFO */
#define TXSTAT_16_ALL   0x00f0	       /* All of the above */

/*
    Bits in the interrupt mask register
*/

#define INT_MASK_TE     0x0080	       /* TX buffer empty */
#define INT_MASK_RF     0x0040	       /* RX buffer full */
#define INT_MASK_TA     0x0020	       /* TX buffer almost empty */
#define INT_MASK_RA     0x0010	       /* RX buffer almost full */
#define INT_MASK_TH     0x0008	       /* TX buffer half empty */
#define INT_MASK_RH     0x0004	       /* RX buffer half full */
#define INT_MASK_TX     0x0002	       /* TX buffer not full */
#define INT_MASK_RX     0x0001	       /* RX buffer not empty */

/*
    Bits we are going to write to interrupt mask register.
*/

#define INTERRUPT_MASK INT_MASK_RX 

/*
    Size of hardware- and driver-level FIFO buffers
*/

#define HARDWARE_BUFFER_SIZE 1024
#define DRIVER_BUFFER_SIZE   (10 * HARDWARE_BUFFER_SIZE)

/*
    Timeout period (in jiffies) for polled reads and writes
*/

#define POLLED_IO_TIMEOUT    1000

/*
    Neat ATOMIC macro.  Use this if you want to atomize a few lines of
    code- if things get big, use a semaphore or something.
*/

#define ATOMIC(code) \
    save_flags(flags) ; \
    cli() ; \
    code ; \
    restore_flags(flags)

/*
    ATOMIC_ON and ATOMIC_OFF.  For bigger ATOMIC sections
*/

#define ATOMIC_ON save_flags(flags) ; cli() 
#define ATOMIC_OFF restore_flags(flags)

/*
    This should be a standard C macro
*/

#define MIN(x,y)   ((x) < (y) ? (x) : (y))

#endif /* TDMB412_CONSTANTS_H */
