/*
    TDMB412 Driver for Linux -- tdmb412-buf.c
    Brian Hughes
    August, 1995
    See ChangeLog for revision history

    Contains the implementation of the input and output buffers
*/

#include <linux/sched.h>
#include <linux/wait.h>
#include <asm/segment.h>
#include <asm/io.h>

#include "tdmb412.h"
#include "tdmb412-const.h"
#include "tdmb412-cfg.h"
#include "tdmb412-xfer.h"
#include "tdmb412-rbuf.h"
#include "tdmb412-vars.h"
#include "tdmb412-xfer.h"

/*
    Local types
*/

struct driver_buffer 
{
    int front ;
    int rear ;
    struct semaphore data_count ;
    char data[DRIVER_BUFFER_SIZE] ;

} ;


/*
    The local buffers and pointer into them.  Array-based queues work like this:

     front == rear                            : queue empty
     front == (rear + 1) % DRIVER_BUFFER_SIZE : queue full
     anything else                            : queue not empty or full
*/

static struct driver_buffer read_buffer = {0, 0, {0, NULL}} ;

/*
    Some macros
*/

#define empty(buf)      ((buf).front == (buf).rear)
#define full(buf)       ((buf).front == ((buf).rear + 1) % DRIVER_BUFFER_SIZE)

/*
    get_data_from_read_buf
        Retrieves data from the kernel-space read buffer and puts it in the
        user-space buffer.  Returns the number of bytes retreived
*/

int get_data_from_read_buf(char *user_buffer,
                           int buffer_size)

{
    int bytes_copied = 0 ;	/* Number of bytes gotten so far */

    while(bytes_copied < buffer_size)
    {
        down(&read_buffer.data_count) ;
        down(&read_buffer.data_count) ;
        put_user(read_buffer.data[read_buffer.rear], (short *)(&user_buffer[bytes_copied]) ) ;
        read_buffer.rear = (read_buffer.rear + 2) % DRIVER_BUFFER_SIZE ;
    }

    #ifdef WATCH_RW
        printk("]") ;
    #endif

    return bytes_copied ;
}

/*
    drain_read_fifo
        Gets data from the read FIFO into the local read buffer until the
        former is empty or the latter is full
*/

void drain_read_fifo(void)

{
    int bytes_read = 0 ;		/* Number of bytes read  */
    unsigned int status ;       /* Status from the board */
    int burst_size ;            /* Number of bytes to get */
    int contiguous_bytes_avail; /* Contiguous space available in buffer */

    down(&device_mutex) ;
    down(&var_mutex) ;

    #ifdef WATCH_RW
        printk("[dr ");
    #endif

    switch(transfer_mode) 
    {
        case TDMB412_16_BIT_XFERS:
            status = inw(RXSTAT_16_REG) ;
            #ifdef WATCH_XFER16_STATUS
                printk("rs%x ", status) ;
            #endif

            while( (status != 0xff30) && !full(read_buffer) ) 
            {
                if(read_buffer.rear < read_buffer.front)
                {
                    contiguous_bytes_avail = read_buffer.rear - 
                                            read_buffer.front ;
                }

                else
                {
                    contiguous_bytes_avail = DRIVER_BUFFER_SIZE - 
                                            read_buffer.rear ;
                }

                burst_size = get_read_burst_size(status) ;
                burst_size = MIN(burst_size, contiguous_bytes_avail) ;
                
                for(bytes_read = 0; bytes_read < burst_size; ++bytes_read)
                {
                    ((short *)read_buffer.data)[read_buffer.front] =
                        inw(RXDATA_16_REG) ;
                    read_buffer.front = (read_buffer.front + 2) %
                                        DRIVER_BUFFER_SIZE ;
                    up(&read_buffer.data_count) ;
                    up(&read_buffer.data_count) ;
                }

                status = inw(RXSTAT_16_REG) ;
                #ifdef WATCH_XFER16_STATUS
                    printk("rs%x ", status) ;
                #endif
            }

            break ;

        case TDMB412_8_BIT_XFERS:
            status = inw(RXSTAT_8_REG) ;
            while( (status != 0) && !full(read_buffer) )
            {
                read_buffer.data[read_buffer.front] = inb(RXDATA_8_REG) ;
                read_buffer.front = (read_buffer.front + 1) %
                                     DRIVER_BUFFER_SIZE ; 
                up(&read_buffer.data_count) ; 
                status = inw(RXSTAT_8_REG) ;
            }

            break ;

        default:
    }

    up(&var_mutex) ;
    up(&device_mutex) ;

    #ifdef WATCH_RW
        printk("]") ;
    #endif
}
