/*
    TDMB412 Driver for Linux -- tdmb412-xfer.c
    Brian Hughes
    August, 1995

    The "workhorse" portions of read and write functions
*/

#include <linux/kernel.h>
#include <linux/sched.h>
#include <asm/segment.h>
#include <asm/io.h>

#include "tdmb412-cfg.h"
#include "tdmb412-const.h"
#include "tdmb412-vars.h"
#include "tdmb412-xfer.h"

/*
    get_16bit_data
        Reads 16-but data from the device and puts in the given
        user-space buffer.  Uses the status bits to minimize the
        polling of the status port. 
*/

int get_16bit_data(char *user_buffer,
                   int size)
{
    int index = 0 ;	           /* Index into user buffer */
    unsigned long int timeout ;	   /* End of timeout period */
    int status = 0 ;	           /* Status from card */
    int burst_size = 0 ;           /* Number of bytes to burst-xfer */
    int count ;	                   /* Number of bytes burst so far */
    
    timeout = jiffies + POLLED_IO_TIMEOUT ;
    
    while( (index < size) && (jiffies < timeout) )
    {	  
        status = inw(RXSTAT_16_REG) ;
        #ifdef WATCH_XFER16_STATUS
            printk("rs%x ", status) ;
        #endif

        burst_size = get_read_burst_size(status) ;
        burst_size = MIN(size - index, burst_size) ;

        #ifdef WATCH_BURST_SIZE
            printk("rbs%i ", burst_size) ;
        #endif

        for(count = 0; count < burst_size; count += 2)
        {
            put_user(inw(RXDATA_16_REG), (short *)(&user_buffer[index]) ) ;
            index += 2 ;
            timeout = jiffies + POLLED_IO_TIMEOUT ;
            #ifdef WATCH_TIMEOUT
                printk("rtl%lu ", timeout - jiffies) ;
            #endif
        }
    }

    if(index != size)
    {
        printk("%s: read timed out with status 0x%x\n", DEVICE_NAME,
                status) ;
    }

    return index ;
}

/*
   get_8bit_data
       Reads 8-bit data from the device and puts it in the given
       user-space buffer
*/

int get_8bit_data(char *user_buffer,
		          int size)
{
    int index = 0 ;		         /* Index into user buffer */
    unsigned long int timeout ;	 /* End of timeout period */
    int status = 0 ;		     /* Status from card */

    timeout = jiffies + POLLED_IO_TIMEOUT ;

    while( (index < size) && (jiffies < timeout) )
    {
        status = inb(RXSTAT_8_REG) ;
	
        if(status != 0)
        {
            put_user(inb(RXDATA_8_REG), &user_buffer[index]) ;
            ++index ;
            #ifdef WATCH_TIMEOUT
	            printk("rtl%lu ", timeout - jiffies) ;
            #endif
	        timeout = jiffies + POLLED_IO_TIMEOUT ;
        }
    }

    if(index != size)
    {
        printk("%s: read times out with status 0x%x\n", DEVICE_NAME,
	            status) ;
    }

    return index ;
}

/*
   put_16bit_data
       Writes 16-bit data to the device from the given user buffer.
       Uses the status bits to minimize status register polling.
*/

int put_16bit_data(const char *user_buffer,
		           int size)
{
    int index = 0 ;		        /* Index into user buffer */
    unsigned long int timeout ;	/* End of timeout period */
    int status = 0 ;		    /* Status from card */
    int burst_size = 0 ;	/* Size of next burst */
    int count ;			    /* Number of bytes sent so far */

    timeout = jiffies + POLLED_IO_TIMEOUT ;

    while( (index < size) && (jiffies < timeout) )
    {
	    status = inw(TXSTAT_16_REG) ;
        #ifdef WATCH_XFER16_STATUS
	        printk("ws%x ", status) ;
        #endif

        burst_size = get_write_burst_size(status) ;
        burst_size = MIN(size - index, burst_size) ;
        
        #ifdef WATCH_BURST_SIZE
            printk("wbs%i ", burst_size) ;
        #endif

        for(count = 0; count < burst_size; count += 2)
        {
            outw(get_user( (short *)(&user_buffer[index]) ), TXDATA_16_REG) ;
            index += 2 ;
            #ifdef WATCH_TIMEOUT
	            printk("wtl%lu ", timeout - jiffies) ;
            #endif
	        timeout = jiffies + POLLED_IO_TIMEOUT ;
        }
    }

    if(index != size)
    {
        printk("%s: write timed out with status 0x%x\n", DEVICE_NAME,
               status) ;
    }

    return index ;
}

/*
   put_8bit_data
       Writes 8-bit data to the device from the given user-space
       buffer
*/

int put_8bit_data(const char *user_buffer,
		          int size)
{
    int index = 0 ;		        /* Index into user buffer */
    unsigned long int timeout ;	/* End of timeout period */
    int status = 0 ;		    /* Status from card */

    timeout = jiffies + POLLED_IO_TIMEOUT ;

    while( (index < size) && (jiffies < timeout) )
    {
        status = inb(TXSTAT_8_REG) ;
	
        if(status != 0)
        {
            outb(get_user(&user_buffer[index]), TXDATA_8_REG) ;
            ++index ;
            #ifdef WATCH_TIMEOUT
                printk("wtl%lu ", timeout - jiffies) ;
            #endif
            timeout = jiffies + POLLED_IO_TIMEOUT ;
	    
        }
    }

    if(index != size)
    {
        printk("%s: read times out with status 0x%x\n", DEVICE_NAME,
               status) ;
    }

    return index ;
}

/*
    get_read_burst_size
        Returns the ideal burst-xfer size based on the given status
*/

inline int get_read_burst_size(int status)

{
    switch(status)
    {
        case 0xff30:
            return 0 ;
            break ;
        case 0xff90:
            return 256 ;
            break ;
        case 0xffb0:
            return 2 ;
            break ;
        case 0xffd0:
            return 512 ;
            break ;
        case 0xffe0:
            return 1024 ;
            break ;
        case 0xfff0:
            return 768 ;
            break ;
        default:
            printk("%s: Got weird 16-bit RX status 0x%x\n", 
                    DEVICE_NAME, status) ;
            if(status & RXSTAT_16_DP)
            {
                return 2 ;
            }

            else
            {
                return 0 ;
            }
    }
}

/*
    get_write_burst_size
        Returns the ideal burst-xfer size based on the given status
*/

inline int get_write_burst_size(int status)

{
    switch(status)
    {
        case 0xff70:
            return 0 ;
            break ;
        case 0xff90:
            return 512 ;
            break ;
        case 0xffa0:
            return 1024 ;
            break ;
        case 0xffb0:
            return 768 ;
            break ;
        case 0xffd0:
            return 256 ;
            break ;
        case 0xfff0:
            return 2 ;
            break ;
        default:
            printk("%s: Got weird 16-bit TX status 0x%x\n", 
                    DEVICE_NAME, status) ;
            if(status & RXSTAT_16_DP)
            {
                return 2 ;
            }

            else
            {
                return 0 ;
            }
    }
}
